# Generated by roxygen2: do not edit by hand

S3method(c,power4test_by_es)
S3method(c,power4test_by_n)
S3method(plot,n_region_from_power)
S3method(plot,power4test_by_es)
S3method(plot,power4test_by_n)
S3method(plot,power_curve)
S3method(plot,q_power_mediation)
S3method(plot,x_from_power)
S3method(predict,power_curve)
S3method(print,n_region_from_power)
S3method(print,power4test)
S3method(print,power4test_by_es)
S3method(print,power4test_by_n)
S3method(print,power_curve)
S3method(print,q_power_mediation)
S3method(print,rejection_rates_df)
S3method(print,sim_data)
S3method(print,sim_out)
S3method(print,summary.n_region_from_power)
S3method(print,summary.x_from_power)
S3method(print,test_out_list)
S3method(print,test_summary)
S3method(print,test_summary_list)
S3method(print,x_from_power)
S3method(rejection_rates,default)
S3method(rejection_rates,n_region_from_power)
S3method(rejection_rates,power4test)
S3method(rejection_rates,power4test_by_es)
S3method(rejection_rates,power4test_by_n)
S3method(rejection_rates,q_power_mediation)
S3method(rejection_rates,x_from_power)
S3method(summary,n_region_from_power)
S3method(summary,q_power_mediation)
S3method(summary,x_from_power)
export(R_for_bz)
export(Rs_bz_supported)
export(arg_x_from_power)
export(as.power4test_by_es)
export(as.power4test_by_n)
export(cut_patterns)
export(do_test)
export(find_par_names)
export(fit_model)
export(gen_boot)
export(gen_mc)
export(missing_values)
export(model_matrices_pop)
export(n_from_power)
export(n_region_from_power)
export(ordinal_variables)
export(pba_diagnosis)
export(pool_sim_data)
export(pop_es_yaml)
export(power4test)
export(power4test_by_es)
export(power4test_by_n)
export(power_curve)
export(ptable_pop)
export(q_power_mediation)
export(q_power_mediation_parallel)
export(q_power_mediation_serial)
export(q_power_mediation_simple)
export(rbeta_rs)
export(rbeta_rs2)
export(rbinary_rs)
export(rejection_rates)
export(rexp_rs)
export(rlnorm_rs)
export(rpgnorm_rs)
export(rt_rs)
export(runif_rs)
export(scale_scores)
export(sim_data)
export(sim_out)
export(summarize_tests)
export(test_cond_indirect)
export(test_cond_indirect_effects)
export(test_group_equal)
export(test_index_of_mome)
export(test_indirect_effect)
export(test_k_indirect_effects)
export(test_moderation)
export(test_parameters)
export(x_from_power)
importFrom(grDevices,adjustcolor)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(graphics,title)
